@@ -12,7 +12,6 @@
 #include "core/fpdfapi/cpdf_pagerendercontext.h"
 #include "core/fpdfapi/page/cpdf_contentparser.h"
 #include "core/fpdfapi/page/cpdf_pageobject.h"
-#include "core/fpdfapi/page/pageint.h"
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_object.h"
@@ -23,44 +22,33 @@
 CPDF_Page::CPDF_Page(CPDF_Document* pDocument,
                      CPDF_Dictionary* pPageDict,
                      bool bPageCache)
-    : m_PageWidth(100),
+    : CPDF_PageObjectHolder(pDocument, pPageDict),
+      m_PageWidth(100),
       m_PageHeight(100),
-      m_pView(nullptr),
-      m_pPageRender(bPageCache ? new CPDF_PageRenderCache(this) : nullptr) {
-  m_pFormDict = pPageDict;
-  m_pDocument = pDocument;
+      m_pView(nullptr) {
+  if (bPageCache)
+    m_pPageRender = pdfium::MakeUnique<CPDF_PageRenderCache>(this);
   if (!pPageDict)
     return;
 
-  CPDF_Object* pageAttr = GetPageAttr("Resources");
-  m_pResources = pageAttr ? pageAttr->GetDict() : nullptr;
+  CPDF_Object* pPageAttr = GetPageAttr("Resources");
+  m_pResources = pPageAttr ? pPageAttr->GetDict() : nullptr;
   m_pPageResources = m_pResources;
-  CPDF_Object* pRotate = GetPageAttr("Rotate");
-  int rotate = pRotate ? pRotate->GetInteger() / 90 % 4 : 0;
-  if (rotate < 0)
-    rotate += 4;
 
-  CPDF_Array* pMediaBox = ToArray(GetPageAttr("MediaBox"));
-  CFX_FloatRect mediabox;
-  if (pMediaBox) {
-    mediabox = pMediaBox->GetRect();
-    mediabox.Normalize();
-  }
+  CFX_FloatRect mediabox = GetBox("MediaBox");
   if (mediabox.IsEmpty())
     mediabox = CFX_FloatRect(0, 0, 612, 792);
 
-  CPDF_Array* pCropBox = ToArray(GetPageAttr("CropBox"));
-  if (pCropBox) {
-    m_BBox = pCropBox->GetRect();
-    m_BBox.Normalize();
-  }
+  m_BBox = GetBox("CropBox");
   if (m_BBox.IsEmpty())
     m_BBox = mediabox;
   else
     m_BBox.Intersect(mediabox);
 
-  m_PageWidth = m_BBox.right - m_BBox.left;
-  m_PageHeight = m_BBox.top - m_BBox.bottom;
+  m_PageWidth = m_BBox.Width();
+  m_PageHeight = m_BBox.Height();
+
+  int rotate = GetPageRotation();
   if (rotate % 2)
     std::swap(m_PageWidth, m_PageHeight);
 
@@ -86,6 +74,10 @@ CPDF_Page::CPDF_Page(CPDF_Document* pDocument,
 
 CPDF_Page::~CPDF_Page() {}
 
+bool CPDF_Page::IsPage() const {
+  return true;
+}
+
 void CPDF_Page::StartParse() {
   if (m_ParseState == CONTENT_PARSED || m_ParseState == CONTENT_PARSING)
     return;
@@ -105,8 +97,8 @@ void CPDF_Page::SetRenderContext(
   m_pRenderContext = std::move(pContext);
 }
 
-CPDF_Object* CPDF_Page::GetPageAttr(const CFX_ByteString& name) const {
-  CPDF_Dictionary* pPageDict = m_pFormDict;
+CPDF_Object* CPDF_Page::GetPageAttr(const ByteString& name) const {
+  CPDF_Dictionary* pPageDict = m_pFormDict.Get();
   std::set<CPDF_Dictionary*> visited;
   while (1) {
     visited.insert(pPageDict);
@@ -120,6 +112,16 @@ CPDF_Object* CPDF_Page::GetPageAttr(const CFX_ByteString& name) const {
   return nullptr;
 }
 
+CFX_FloatRect CPDF_Page::GetBox(const ByteString& name) const {
+  CFX_FloatRect box;
+  CPDF_Array* pBox = ToArray(GetPageAttr(name));
+  if (pBox) {
+    box = pBox->GetRect();
+    box.Normalize();
+  }
+  return box;
+}
+
 CFX_Matrix CPDF_Page::GetDisplayMatrix(int xPos,
                                        int yPos,
                                        int xSize,
@@ -176,10 +178,47 @@ CFX_Matrix CPDF_Page::GetDisplayMatrix(int xPos,
   return matrix;
 }
 
+// This method follows the same apparent logic as GetDisplayMatrix(). For
+// example, consider point 0. First, take the top left coordinate of the
+// rectangle in the transformed space. Now, calculate what this point was in the
+// original space by inverting. Note that this is not necessarily the same as
+// the top left corner of the original rectangle.
+CFX_Matrix CPDF_Page::GetDisplayMatrixWithTransformation(
+    int xPos,
+    int yPos,
+    int xSize,
+    int ySize,
+    const CFX_Matrix& transformation) {
+  CFX_FloatRect rect(xPos, yPos, xPos + xSize, yPos + ySize);
+  rect = transformation.TransformRect(rect);
+  CFX_Matrix inverse = transformation.GetInverse();
+  CFX_PointF point0(rect.left, rect.top);
+  CFX_PointF point1(rect.left, rect.bottom);
+  CFX_PointF point2(rect.right, rect.top);
+  point0 = inverse.Transform(point0);
+  point1 = inverse.Transform(point1);
+  point2 = inverse.Transform(point2);
+
+  CFX_Matrix matrix = m_PageMatrix;
+  matrix.Concat(CFX_Matrix(
+      (point2.x - point0.x) / m_PageWidth, (point2.y - point0.y) / m_PageWidth,
+      (point1.x - point0.x) / m_PageHeight,
+      (point1.y - point0.y) / m_PageHeight, point0.x, point0.y));
+  return matrix;
+}
+
+int CPDF_Page::GetPageRotation() const {
+  CPDF_Object* pRotate = GetPageAttr("Rotate");
+  int rotate = pRotate ? (pRotate->GetInteger() / 90) % 4 : 0;
+  return (rotate < 0) ? (rotate + 4) : rotate;
+}
+
 bool GraphicsData::operator<(const GraphicsData& other) const {
   if (fillAlpha != other.fillAlpha)
     return fillAlpha < other.fillAlpha;
-  return strokeAlpha < other.strokeAlpha;
+  if (strokeAlpha != other.strokeAlpha)
+    return strokeAlpha < other.strokeAlpha;
+  return blendType < other.blendType;
 }
 
 bool FontData::operator<(const FontData& other) const {
