@@ -25,7 +25,6 @@
 #include "core/fpdfapi/page/cpdf_shadingpattern.h"
 #include "core/fpdfapi/page/cpdf_streamparser.h"
 #include "core/fpdfapi/page/cpdf_textobject.h"
-#include "core/fpdfapi/page/pageint.h"
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_document.h"
@@ -36,7 +35,9 @@
 #include "core/fpdfapi/parser/fpdf_parser_decode.h"
 #include "core/fxcrt/fx_safe_types.h"
 #include "core/fxge/cfx_graphstatedata.h"
+#include "third_party/base/logging.h"
 #include "third_party/base/ptr_util.h"
+#include "third_party/base/stl_util.h"
 
 namespace {
 
@@ -75,11 +76,11 @@ CFX_FloatRect GetShadingBBox(CPDF_ShadingPattern* pShading,
   CPDF_Stream* pStream = ToStream(pShading->GetShadingObject());
   CPDF_ColorSpace* pCS = pShading->GetCS();
   if (!pStream || !pCS)
-    return CFX_FloatRect(0, 0, 0, 0);
+    return CFX_FloatRect();
 
   CPDF_MeshStream stream(type, pShading->GetFuncs(), pStream, pCS);
   if (!stream.Load())
-    return CFX_FloatRect(0, 0, 0, 0);
+    return CFX_FloatRect();
 
   CFX_FloatRect rect;
   bool bStarted = false;
@@ -114,9 +115,9 @@ CFX_FloatRect GetShadingBBox(CPDF_ShadingPattern* pShading,
         break;
       CFX_PointF origin = stream.ReadCoords();
       if (bStarted) {
-        rect.UpdateRect(origin.x, origin.y);
+        rect.UpdateRect(origin);
       } else {
-        rect.InitRect(origin.x, origin.y);
+        rect.InitRect(origin);
         bStarted = true;
       }
     }
@@ -130,8 +131,7 @@ CFX_FloatRect GetShadingBBox(CPDF_ShadingPattern* pShading,
     if (bGouraud)
       stream.BitStream()->ByteAlign();
   }
-  matrix.TransformRect(rect);
-  return rect;
+  return matrix.TransformRect(rect);
 }
 
 struct AbbrPair {
@@ -156,18 +156,17 @@ const AbbrPair InlineValueAbbr[] = {
 
 struct AbbrReplacementOp {
   bool is_replace_key;
-  CFX_ByteString key;
-  CFX_ByteStringC replacement;
+  ByteString key;
+  ByteStringView replacement;
 };
 
-CFX_ByteStringC FindFullName(const AbbrPair* table,
-                             size_t count,
-                             const CFX_ByteStringC& abbr) {
+ByteStringView FindFullName(const AbbrPair* table,
+                            size_t count,
+                            const ByteStringView& abbr) {
   auto* it = std::find_if(table, table + count, [abbr](const AbbrPair& pair) {
     return pair.abbr == abbr;
   });
-  return it != table + count ? CFX_ByteStringC(it->full_name)
-                             : CFX_ByteStringC();
+  return it != table + count ? ByteStringView(it->full_name) : ByteStringView();
 }
 
 void ReplaceAbbr(CPDF_Object* pObj) {
@@ -176,10 +175,10 @@ void ReplaceAbbr(CPDF_Object* pObj) {
       CPDF_Dictionary* pDict = pObj->AsDictionary();
       std::vector<AbbrReplacementOp> replacements;
       for (const auto& it : *pDict) {
-        CFX_ByteString key = it.first;
+        ByteString key = it.first;
         CPDF_Object* value = it.second.get();
-        CFX_ByteStringC fullname = FindFullName(
-            InlineKeyAbbr, FX_ArraySize(InlineKeyAbbr), key.AsStringC());
+        ByteStringView fullname = FindFullName(
+            InlineKeyAbbr, FX_ArraySize(InlineKeyAbbr), key.AsStringView());
         if (!fullname.IsEmpty()) {
           AbbrReplacementOp op;
           op.is_replace_key = true;
@@ -190,9 +189,10 @@ void ReplaceAbbr(CPDF_Object* pObj) {
         }
 
         if (value->IsName()) {
-          CFX_ByteString name = value->GetString();
-          fullname = FindFullName(
-              InlineValueAbbr, FX_ArraySize(InlineValueAbbr), name.AsStringC());
+          ByteString name = value->GetString();
+          fullname =
+              FindFullName(InlineValueAbbr, FX_ArraySize(InlineValueAbbr),
+                           name.AsStringView());
           if (!fullname.IsEmpty()) {
             AbbrReplacementOp op;
             op.is_replace_key = false;
@@ -206,9 +206,9 @@ void ReplaceAbbr(CPDF_Object* pObj) {
       }
       for (const auto& op : replacements) {
         if (op.is_replace_key)
-          pDict->ReplaceKey(op.key, CFX_ByteString(op.replacement));
+          pDict->ReplaceKey(op.key, ByteString(op.replacement));
         else
-          pDict->SetNewFor<CPDF_Name>(op.key, CFX_ByteString(op.replacement));
+          pDict->SetNewFor<CPDF_Name>(op.key, ByteString(op.replacement));
       }
       break;
     }
@@ -217,11 +217,12 @@ void ReplaceAbbr(CPDF_Object* pObj) {
       for (size_t i = 0; i < pArray->GetCount(); i++) {
         CPDF_Object* pElement = pArray->GetObjectAt(i);
         if (pElement->IsName()) {
-          CFX_ByteString name = pElement->GetString();
-          CFX_ByteStringC fullname = FindFullName(
-              InlineValueAbbr, FX_ArraySize(InlineValueAbbr), name.AsStringC());
+          ByteString name = pElement->GetString();
+          ByteStringView fullname =
+              FindFullName(InlineValueAbbr, FX_ArraySize(InlineValueAbbr),
+                           name.AsStringView());
           if (!fullname.IsEmpty())
-            pArray->SetNewAt<CPDF_Name>(i, CFX_ByteString(fullname));
+            pArray->SetNewAt<CPDF_Name>(i, ByteString(fullname));
         } else {
           ReplaceAbbr(pElement);
         }
@@ -235,15 +236,6 @@ void ReplaceAbbr(CPDF_Object* pObj) {
 
 }  // namespace
 
-CFX_ByteStringC PDF_FindKeyAbbreviationForTesting(const CFX_ByteStringC& abbr) {
-  return FindFullName(InlineKeyAbbr, FX_ArraySize(InlineKeyAbbr), abbr);
-}
-
-CFX_ByteStringC PDF_FindValueAbbreviationForTesting(
-    const CFX_ByteStringC& abbr) {
-  return FindFullName(InlineValueAbbr, FX_ArraySize(InlineValueAbbr), abbr);
-}
-
 CPDF_StreamContentParser::CPDF_StreamContentParser(
     CPDF_Document* pDocument,
     CPDF_Dictionary* pPageResources,
@@ -251,15 +243,16 @@ CPDF_StreamContentParser::CPDF_StreamContentParser(
     const CFX_Matrix* pmtContentToUser,
     CPDF_PageObjectHolder* pObjHolder,
     CPDF_Dictionary* pResources,
-    CFX_FloatRect* pBBox,
+    const CFX_FloatRect& rcBBox,
     CPDF_AllStates* pStates,
-    int level)
+    std::set<const uint8_t*>* parsedSet)
     : m_pDocument(pDocument),
       m_pPageResources(pPageResources),
       m_pParentResources(pParentResources),
       m_pResources(pResources),
       m_pObjectHolder(pObjHolder),
-      m_Level(level),
+      m_ParsedSet(parsedSet),
+      m_BBox(rcBBox),
       m_ParamStartPos(0),
       m_ParamCount(0),
       m_pCurStates(pdfium::MakeUnique<CPDF_AllStates>()),
@@ -279,8 +272,6 @@ CPDF_StreamContentParser::CPDF_StreamContentParser(
     m_pResources = m_pParentResources;
   if (!m_pResources)
     m_pResources = m_pPageResources;
-  if (pBBox)
-    m_BBox = *pBBox;
   if (pStates) {
     m_pCurStates->Copy(*pStates);
   } else {
@@ -317,7 +308,7 @@ int CPDF_StreamContentParser::GetNextParamPos() {
   return index;
 }
 
-void CPDF_StreamContentParser::AddNameParam(const CFX_ByteStringC& bsName) {
+void CPDF_StreamContentParser::AddNameParam(const ByteStringView& bsName) {
   ContentParam& param = m_ParamBuf[GetNextParamPos()];
   if (bsName.GetLength() > 32) {
     param.m_Type = ContentParam::OBJECT;
@@ -325,18 +316,18 @@ void CPDF_StreamContentParser::AddNameParam(const CFX_ByteStringC& bsName) {
         m_pDocument->GetByteStringPool(), PDF_NameDecode(bsName));
   } else {
     param.m_Type = ContentParam::NAME;
-    if (bsName.Find('#') == -1) {
-      memcpy(param.m_Name.m_Buffer, bsName.raw_str(), bsName.GetLength());
-      param.m_Name.m_Len = bsName.GetLength();
-    } else {
-      CFX_ByteString str = PDF_NameDecode(bsName);
+    if (bsName.Contains('#')) {
+      ByteString str = PDF_NameDecode(bsName);
       memcpy(param.m_Name.m_Buffer, str.c_str(), str.GetLength());
       param.m_Name.m_Len = str.GetLength();
+    } else {
+      memcpy(param.m_Name.m_Buffer, bsName.raw_str(), bsName.GetLength());
+      param.m_Name.m_Len = bsName.GetLength();
     }
   }
 }
 
-void CPDF_StreamContentParser::AddNumberParam(const CFX_ByteStringC& str) {
+void CPDF_StreamContentParser::AddNumberParam(const ByteStringView& str) {
   ContentParam& param = m_ParamBuf[GetNextParamPos()];
   param.m_Type = ContentParam::NUMBER;
   param.m_Number.m_bInteger = FX_atonum(str, &param.m_Number.m_Integer);
@@ -383,19 +374,19 @@ CPDF_Object* CPDF_StreamContentParser::GetObject(uint32_t index) {
     param.m_Type = ContentParam::OBJECT;
     param.m_pObject = pdfium::MakeUnique<CPDF_Name>(
         m_pDocument->GetByteStringPool(),
-        CFX_ByteString(param.m_Name.m_Buffer, param.m_Name.m_Len));
+        ByteString(param.m_Name.m_Buffer, param.m_Name.m_Len));
     return param.m_pObject.get();
   }
   if (param.m_Type == ContentParam::OBJECT)
     return param.m_pObject.get();
 
-  ASSERT(false);
+  NOTREACHED();
   return nullptr;
 }
 
-CFX_ByteString CPDF_StreamContentParser::GetString(uint32_t index) {
+ByteString CPDF_StreamContentParser::GetString(uint32_t index) {
   if (index >= m_ParamCount) {
-    return CFX_ByteString();
+    return ByteString();
   }
   int real_index = m_ParamStartPos + m_ParamCount - index - 1;
   if (real_index >= kParamBufSize) {
@@ -403,12 +394,12 @@ CFX_ByteString CPDF_StreamContentParser::GetString(uint32_t index) {
   }
   ContentParam& param = m_ParamBuf[real_index];
   if (param.m_Type == ContentParam::NAME) {
-    return CFX_ByteString(param.m_Name.m_Buffer, param.m_Name.m_Len);
+    return ByteString(param.m_Name.m_Buffer, param.m_Name.m_Len);
   }
   if (param.m_Type == 0 && param.m_pObject) {
     return param.m_pObject->GetString();
   }
-  return CFX_ByteString();
+  return ByteString();
 }
 
 float CPDF_StreamContentParser::GetNumber(uint32_t index) {
@@ -569,7 +560,7 @@ CPDF_StreamContentParser::InitializeOpCodes() {
   });
 }
 
-void CPDF_StreamContentParser::OnOperator(const CFX_ByteStringC& op) {
+void CPDF_StreamContentParser::OnOperator(const ByteStringView& op) {
   static const OpCodes s_OpCodes = InitializeOpCodes();
 
   auto it = s_OpCodes.find(op.GetID());
@@ -596,7 +587,7 @@ void CPDF_StreamContentParser::Handle_EOFillStrokePath() {
 }
 
 void CPDF_StreamContentParser::Handle_BeginMarkedContent_Dictionary() {
-  CFX_ByteString tag = GetString(1);
+  ByteString tag = GetString(1);
   CPDF_Object* pProperty = GetObject(0);
   if (!pProperty) {
     return;
@@ -628,12 +619,13 @@ void CPDF_StreamContentParser::Handle_BeginImage() {
     if (type != CPDF_StreamParser::Name) {
       break;
     }
-    CFX_ByteString key(m_pSyntax->GetWord().Mid(1));
+    auto word = m_pSyntax->GetWord();
+    ByteString key(word.Right(word.GetLength() - 1));
     auto pObj = m_pSyntax->ReadNextObject(false, false, 0);
     if (!key.IsEmpty()) {
       uint32_t dwObjNum = pObj ? pObj->GetObjNum() : 0;
       if (dwObjNum)
-        pDict->SetNewFor<CPDF_Reference>(key, m_pDocument, dwObjNum);
+        pDict->SetNewFor<CPDF_Reference>(key, m_pDocument.Get(), dwObjNum);
       else
         pDict->SetFor(key, std::move(pObj));
     }
@@ -643,7 +635,7 @@ void CPDF_StreamContentParser::Handle_BeginImage() {
   if (pDict->KeyExist("ColorSpace")) {
     pCSObj = pDict->GetDirectObjectFor("ColorSpace");
     if (pCSObj->IsName()) {
-      CFX_ByteString name = pCSObj->GetString();
+      ByteString name = pCSObj->GetString();
       if (name != "DeviceRGB" && name != "DeviceGray" && name != "DeviceCMYK") {
         pCSObj = FindResourceObj("ColorSpace", name);
         if (pCSObj && pCSObj->IsInline())
@@ -653,7 +645,7 @@ void CPDF_StreamContentParser::Handle_BeginImage() {
   }
   pDict->SetNewFor<CPDF_Name>("Subtype", "Image");
   std::unique_ptr<CPDF_Stream> pStream =
-      m_pSyntax->ReadInlineStream(m_pDocument, std::move(pDict), pCSObj);
+      m_pSyntax->ReadInlineStream(m_pDocument.Get(), std::move(pDict), pCSObj);
   while (1) {
     CPDF_StreamParser::SyntaxType type = m_pSyntax->ParseNextElement();
     if (type == CPDF_StreamParser::EndOfData) {
@@ -666,7 +658,11 @@ void CPDF_StreamContentParser::Handle_BeginImage() {
       break;
     }
   }
-  AddImage(std::move(pStream));
+  CPDF_ImageObject* pObj = AddImage(std::move(pStream));
+  // Record the bounding box of this image, so rendering code can draw it
+  // properly.
+  if (pObj && pObj->GetImage()->IsMask())
+    m_pObjectHolder->AddImageMaskBoundingBox(pObj->GetRect());
 }
 
 void CPDF_StreamContentParser::Handle_BeginMarkedContent() {
@@ -732,10 +728,14 @@ void CPDF_StreamContentParser::Handle_SetCachedDevice() {
 }
 
 void CPDF_StreamContentParser::Handle_ExecuteXObject() {
-  CFX_ByteString name = GetString(0);
+  ByteString name = GetString(0);
   if (name == m_LastImageName && m_pLastImage && m_pLastImage->GetStream() &&
       m_pLastImage->GetStream()->GetObjNum()) {
-    AddImage(m_pLastImage);
+    CPDF_ImageObject* pObj = AddImage(m_pLastImage);
+    // Record the bounding box of this image, so rendering code can draw it
+    // properly.
+    if (pObj && pObj->GetImage()->IsMask())
+      m_pObjectHolder->AddImageMaskBoundingBox(pObj->GetRect());
     return;
   }
 
@@ -745,7 +745,7 @@ void CPDF_StreamContentParser::Handle_ExecuteXObject() {
     return;
   }
 
-  CFX_ByteString type;
+  ByteString type;
   if (pXObject->GetDict())
     type = pXObject->GetDict()->GetStringFor("Subtype");
 
@@ -756,9 +756,11 @@ void CPDF_StreamContentParser::Handle_ExecuteXObject() {
                                  : AddImage(pXObject->GetObjNum());
 
     m_LastImageName = name;
-    m_pLastImage = pObj->GetImage();
-    if (!m_pObjectHolder->HasImageMask())
-      m_pObjectHolder->SetHasImageMask(m_pLastImage->IsMask());
+    if (pObj) {
+      m_pLastImage = pObj->GetImage();
+      if (m_pLastImage->IsMask())
+        m_pObjectHolder->AddImageMaskBoundingBox(pObj->GetRect());
+    }
   } else if (type == "Form") {
     AddForm(pXObject);
   }
@@ -767,7 +769,7 @@ void CPDF_StreamContentParser::Handle_ExecuteXObject() {
 void CPDF_StreamContentParser::AddForm(CPDF_Stream* pStream) {
   auto pFormObj = pdfium::MakeUnique<CPDF_FormObject>();
   pFormObj->m_pForm = pdfium::MakeUnique<CPDF_Form>(
-      m_pDocument, m_pPageResources, pStream, m_pResources);
+      m_pDocument.Get(), m_pPageResources.Get(), pStream, m_pResources.Get());
   pFormObj->m_FormMatrix = m_pCurStates->m_CTM;
   pFormObj->m_FormMatrix.Concat(m_mtContentToUser);
   CPDF_AllStates status;
@@ -775,7 +777,8 @@ void CPDF_StreamContentParser::AddForm(CPDF_Stream* pStream) {
   status.m_GraphState = m_pCurStates->m_GraphState;
   status.m_ColorState = m_pCurStates->m_ColorState;
   status.m_TextState = m_pCurStates->m_TextState;
-  pFormObj->m_pForm->ParseContent(&status, nullptr, nullptr, m_Level + 1);
+  pFormObj->m_pForm->ParseContentWithParams(&status, nullptr, nullptr,
+                                            m_ParsedSet.Get());
   if (!m_pObjectHolder->BackgroundAlphaNeeded() &&
       pFormObj->m_pForm->BackgroundAlphaNeeded()) {
     m_pObjectHolder->SetBackgroundAlphaNeeded(true);
@@ -792,7 +795,7 @@ CPDF_ImageObject* CPDF_StreamContentParser::AddImage(
 
   auto pImageObj = pdfium::MakeUnique<CPDF_ImageObject>();
   pImageObj->SetImage(
-      pdfium::MakeRetain<CPDF_Image>(m_pDocument, std::move(pStream)));
+      pdfium::MakeRetain<CPDF_Image>(m_pDocument.Get(), std::move(pStream)));
   return AddImageObject(std::move(pImageObj));
 }
 
@@ -803,7 +806,7 @@ CPDF_ImageObject* CPDF_StreamContentParser::AddImage(uint32_t streamObjNum) {
 }
 
 CPDF_ImageObject* CPDF_StreamContentParser::AddImage(
-    const CFX_RetainPtr<CPDF_Image>& pImage) {
+    const RetainPtr<CPDF_Image>& pImage) {
   if (!pImage)
     return nullptr;
 
@@ -834,7 +837,7 @@ void CPDF_StreamContentParser::Handle_MarkPlace_Dictionary() {}
 void CPDF_StreamContentParser::Handle_EndImage() {}
 
 void CPDF_StreamContentParser::Handle_EndMarkedContent() {
-  if (m_CurContentMark)
+  if (m_CurContentMark.HasRef())
     m_CurContentMark.DeleteLastMark();
 }
 
@@ -873,7 +876,7 @@ void CPDF_StreamContentParser::Handle_SetGray_Stroke() {
 }
 
 void CPDF_StreamContentParser::Handle_SetExtendGraphState() {
-  CFX_ByteString name = GetString(0);
+  ByteString name = GetString(0);
   CPDF_Dictionary* pGS = ToDictionary(FindResourceObj("ExtGState", name));
   if (!pGS) {
     m_bResourceMissing = true;
@@ -1117,7 +1120,7 @@ void CPDF_StreamContentParser::Handle_ShadeFill() {
   pObj->m_Matrix = m_pCurStates->m_CTM;
   pObj->m_Matrix.Concat(m_mtContentToUser);
   CFX_FloatRect bbox =
-      pObj->m_ClipPath ? pObj->m_ClipPath.GetClipBox() : m_BBox;
+      pObj->m_ClipPath.HasRef() ? pObj->m_ClipPath.GetClipBox() : m_BBox;
   if (pShading->IsMeshShading())
     bbox.Intersect(GetShadingBBox(pShading, pObj->m_Matrix));
   pObj->m_Left = bbox.left;
@@ -1153,9 +1156,8 @@ void CPDF_StreamContentParser::Handle_SetFont() {
   }
 }
 
-CPDF_Object* CPDF_StreamContentParser::FindResourceObj(
-    const CFX_ByteString& type,
-    const CFX_ByteString& name) {
+CPDF_Object* CPDF_StreamContentParser::FindResourceObj(const ByteString& type,
+                                                       const ByteString& name) {
   if (!m_pResources)
     return nullptr;
   CPDF_Dictionary* pDict = m_pResources->GetDictFor(type);
@@ -1168,29 +1170,29 @@ CPDF_Object* CPDF_StreamContentParser::FindResourceObj(
   return pPageDict ? pPageDict->GetDirectObjectFor(name) : nullptr;
 }
 
-CPDF_Font* CPDF_StreamContentParser::FindFont(const CFX_ByteString& name) {
+CPDF_Font* CPDF_StreamContentParser::FindFont(const ByteString& name) {
   CPDF_Dictionary* pFontDict = ToDictionary(FindResourceObj("Font", name));
   if (!pFontDict) {
     m_bResourceMissing = true;
-    return CPDF_Font::GetStockFont(m_pDocument, "Helvetica");
+    return CPDF_Font::GetStockFont(m_pDocument.Get(), "Helvetica");
   }
 
   CPDF_Font* pFont = m_pDocument->LoadFont(pFontDict);
   if (pFont && pFont->IsType3Font()) {
-    pFont->AsType3Font()->SetPageResources(m_pResources);
+    pFont->AsType3Font()->SetPageResources(m_pResources.Get());
     pFont->AsType3Font()->CheckType3FontMetrics();
   }
   return pFont;
 }
 
 CPDF_ColorSpace* CPDF_StreamContentParser::FindColorSpace(
-    const CFX_ByteString& name) {
+    const ByteString& name) {
   if (name == "Pattern") {
     return CPDF_ColorSpace::GetStockCS(PDFCS_PATTERN);
   }
   if (name == "DeviceGray" || name == "DeviceCMYK" || name == "DeviceRGB") {
-    CFX_ByteString defname = "Default";
-    defname += name.Mid(7);
+    ByteString defname = "Default";
+    defname += name.Right(name.GetLength() - 7);
     CPDF_Object* pDefObj = FindResourceObj("ColorSpace", defname);
     if (!pDefObj) {
       if (name == "DeviceGray") {
@@ -1211,7 +1213,7 @@ CPDF_ColorSpace* CPDF_StreamContentParser::FindColorSpace(
   return m_pDocument->LoadColorSpace(pCSObj);
 }
 
-CPDF_Pattern* CPDF_StreamContentParser::FindPattern(const CFX_ByteString& name,
+CPDF_Pattern* CPDF_StreamContentParser::FindPattern(const ByteString& name,
                                                     bool bShading) {
   CPDF_Object* pPattern =
       FindResourceObj(bShading ? "Shading" : "Pattern", name);
@@ -1223,7 +1225,7 @@ CPDF_Pattern* CPDF_StreamContentParser::FindPattern(const CFX_ByteString& name,
                                   m_pCurStates->m_ParentMatrix);
 }
 
-void CPDF_StreamContentParser::AddTextObject(CFX_ByteString* pStrs,
+void CPDF_StreamContentParser::AddTextObject(ByteString* pStrs,
                                              float fInitKerning,
                                              float* pKerning,
                                              int nsegs) {
@@ -1251,7 +1253,7 @@ void CPDF_StreamContentParser::AddTextObject(CFX_ByteString* pStrs,
   {
     auto pText = pdfium::MakeUnique<CPDF_TextObject>();
     m_pLastTextObject = pText.get();
-    SetGraphicStates(m_pLastTextObject, true, true, true);
+    SetGraphicStates(m_pLastTextObject.Get(), true, true, true);
     if (TextRenderingModeIsStrokeMode(text_mode)) {
       float* pCTM = pText->m_TextState.GetMutableCTM();
       pCTM[0] = m_pCurStates->m_CTM.a;
@@ -1260,10 +1262,11 @@ void CPDF_StreamContentParser::AddTextObject(CFX_ByteString* pStrs,
       pCTM[3] = m_pCurStates->m_CTM.d;
     }
     pText->SetSegments(pStrs, pKerning, nsegs);
-    pText->m_Pos = m_mtContentToUser.Transform(
-        m_pCurStates->m_CTM.Transform(m_pCurStates->m_TextMatrix.Transform(
-            CFX_PointF(m_pCurStates->m_TextPos.x,
-                       m_pCurStates->m_TextPos.y + m_pCurStates->m_TextRise))));
+    pText->SetPosition(
+        m_mtContentToUser.Transform(m_pCurStates->m_CTM.Transform(
+            m_pCurStates->m_TextMatrix.Transform(CFX_PointF(
+                m_pCurStates->m_TextPos.x,
+                m_pCurStates->m_TextPos.y + m_pCurStates->m_TextRise)))));
 
     m_pCurStates->m_TextPos +=
         pText->CalcPositionData(m_pCurStates->m_TextHorzScale);
@@ -1288,7 +1291,7 @@ void CPDF_StreamContentParser::AddTextObject(CFX_ByteString* pStrs,
 }
 
 void CPDF_StreamContentParser::Handle_ShowText() {
-  CFX_ByteString str = GetString(0);
+  ByteString str = GetString(0);
   if (str.IsEmpty()) {
     return;
   }
@@ -1315,31 +1318,27 @@ void CPDF_StreamContentParser::Handle_ShowText_Positioning() {
     }
     return;
   }
-  CFX_ByteString* pStrs = new CFX_ByteString[nsegs];
-  float* pKerning = FX_Alloc(float, nsegs);
+  std::vector<ByteString> strs(nsegs);
+  std::vector<float> kernings(nsegs);
   size_t iSegment = 0;
   float fInitKerning = 0;
   for (size_t i = 0; i < n; i++) {
     CPDF_Object* pObj = pArray->GetDirectObjectAt(i);
     if (pObj->IsString()) {
-      CFX_ByteString str = pObj->GetString();
-      if (str.IsEmpty()) {
+      ByteString str = pObj->GetString();
+      if (str.IsEmpty())
         continue;
-      }
-      pStrs[iSegment] = str;
-      pKerning[iSegment++] = 0;
+      strs[iSegment] = str;
+      kernings[iSegment++] = 0;
     } else {
-      float num = pObj ? pObj->GetNumber() : 0;
-      if (iSegment == 0) {
+      float num = pObj->GetNumber();
+      if (iSegment == 0)
         fInitKerning += num;
-      } else {
-        pKerning[iSegment - 1] += num;
-      }
+      else
+        kernings[iSegment - 1] += num;
     }
   }
-  AddTextObject(pStrs, fInitKerning, pKerning, iSegment);
-  delete[] pStrs;
-  FX_Free(pKerning);
+  AddTextObject(strs.data(), fInitKerning, kernings.data(), iSegment);
 }
 
 void CPDF_StreamContentParser::Handle_SetTextLeading() {
@@ -1510,9 +1509,13 @@ void CPDF_StreamContentParser::AddPathObject(int FillType, bool bStroke) {
 uint32_t CPDF_StreamContentParser::Parse(const uint8_t* pData,
                                          uint32_t dwSize,
                                          uint32_t max_cost) {
-  if (m_Level > kMaxFormLevel)
+  if (m_ParsedSet->size() > kMaxFormLevel ||
+      pdfium::ContainsKey(*m_ParsedSet, pData))
     return dwSize;
 
+  pdfium::ScopedSetInsertion<const uint8_t*> scopedInsert(m_ParsedSet.Get(),
+                                                          pData);
+
   uint32_t InitObjCount = m_pObjectHolder->GetPageObjectList()->size();
   CPDF_StreamParser syntax(pData, dwSize, m_pDocument->GetByteStringPool());
   CPDF_StreamParserAutoClearer auto_clearer(&m_pSyntax, &syntax);
@@ -1531,9 +1534,11 @@ uint32_t CPDF_StreamContentParser::Parse(const uint8_t* pData,
       case CPDF_StreamParser::Number:
         AddNumberParam(syntax.GetWord());
         break;
-      case CPDF_StreamParser::Name:
-        AddNameParam(syntax.GetWord().Mid(1));
+      case CPDF_StreamParser::Name: {
+        auto word = syntax.GetWord();
+        AddNameParam(word.Right(word.GetLength() - 1));
         break;
+      }
       default:
         AddObjectParam(syntax.GetObject());
     }
@@ -1552,7 +1557,7 @@ void CPDF_StreamContentParser::ParsePathObject() {
       case CPDF_StreamParser::EndOfData:
         return;
       case CPDF_StreamParser::Keyword: {
-        CFX_ByteStringC strc = m_pSyntax->GetWord();
+        ByteStringView strc = m_pSyntax->GetWord();
         int len = strc.GetLength();
         if (len == 1) {
           switch (strc[0]) {
@@ -1626,6 +1631,18 @@ void CPDF_StreamContentParser::ParsePathObject() {
   }
 }
 
+// static
+ByteStringView CPDF_StreamContentParser::FindKeyAbbreviationForTesting(
+    const ByteStringView& abbr) {
+  return FindFullName(InlineKeyAbbr, FX_ArraySize(InlineKeyAbbr), abbr);
+}
+
+// static
+ByteStringView CPDF_StreamContentParser::FindValueAbbreviationForTesting(
+    const ByteStringView& abbr) {
+  return FindFullName(InlineValueAbbr, FX_ArraySize(InlineValueAbbr), abbr);
+}
+
 CPDF_StreamContentParser::ContentParam::ContentParam() {}
 
 CPDF_StreamContentParser::ContentParam::~ContentParam() {}
