@@ -13,16 +13,13 @@
 #include "core/fxcodec/jbig2/JBig2_PatternDict.h"
 #include "third_party/base/ptr_util.h"
 
-CJBig2_PatternDict* CJBig2_PDDProc::decode_Arith(
+std::unique_ptr<CJBig2_PatternDict> CJBig2_PDDProc::decode_Arith(
     CJBig2_ArithDecoder* pArithDecoder,
     JBig2ArithCtx* gbContext,
-    IFX_Pause* pPause) {
+    IFX_PauseIndicator* pPause) {
   uint32_t GRAY;
-  CJBig2_Image* BHDC = nullptr;
-  auto pDict = pdfium::MakeUnique<CJBig2_PatternDict>();
-  pDict->NUMPATS = GRAYMAX + 1;
-  pDict->HDPATS = FX_Alloc(CJBig2_Image*, pDict->NUMPATS);
-  JBIG2_memset(pDict->HDPATS, 0, sizeof(CJBig2_Image*) * pDict->NUMPATS);
+  std::unique_ptr<CJBig2_Image> BHDC;
+  auto pDict = pdfium::MakeUnique<CJBig2_PatternDict>(GRAYMAX + 1);
 
   auto pGRD = pdfium::MakeUnique<CJBig2_GRDProc>();
   pGRD->MMR = HDMMR;
@@ -44,7 +41,7 @@ CJBig2_PatternDict* CJBig2_PDDProc::decode_Arith(
   FXCODEC_STATUS status =
       pGRD->Start_decode_Arith(&BHDC, pArithDecoder, gbContext, nullptr);
   while (status == FXCODEC_STATUS_DECODE_TOBECONTINUE)
-    status = pGRD->Continue_decode(pPause);
+    status = pGRD->Continue_decode(pPause, pArithDecoder);
   if (!BHDC)
     return nullptr;
 
@@ -53,24 +50,20 @@ CJBig2_PatternDict* CJBig2_PDDProc::decode_Arith(
     pDict->HDPATS[GRAY] = BHDC->subImage(HDPW * GRAY, 0, HDPW, HDPH);
     GRAY = GRAY + 1;
   }
-  delete BHDC;
-  return pDict.release();
+  return pDict;
 }
 
-CJBig2_PatternDict* CJBig2_PDDProc::decode_MMR(CJBig2_BitStream* pStream,
-                                               IFX_Pause* pPause) {
+std::unique_ptr<CJBig2_PatternDict> CJBig2_PDDProc::decode_MMR(
+    CJBig2_BitStream* pStream) {
   uint32_t GRAY;
-  CJBig2_Image* BHDC = nullptr;
-  auto pDict = pdfium::MakeUnique<CJBig2_PatternDict>();
-  pDict->NUMPATS = GRAYMAX + 1;
-  pDict->HDPATS = FX_Alloc(CJBig2_Image*, pDict->NUMPATS);
-  JBIG2_memset(pDict->HDPATS, 0, sizeof(CJBig2_Image*) * pDict->NUMPATS);
+  std::unique_ptr<CJBig2_Image> BHDC;
+  auto pDict = pdfium::MakeUnique<CJBig2_PatternDict>(GRAYMAX + 1);
 
   auto pGRD = pdfium::MakeUnique<CJBig2_GRDProc>();
   pGRD->MMR = HDMMR;
   pGRD->GBW = (GRAYMAX + 1) * HDPW;
   pGRD->GBH = HDPH;
-  pGRD->Start_decode_MMR(&BHDC, pStream, nullptr);
+  pGRD->Start_decode_MMR(&BHDC, pStream);
   if (!BHDC)
     return nullptr;
 
@@ -79,6 +72,5 @@ CJBig2_PatternDict* CJBig2_PDDProc::decode_MMR(CJBig2_BitStream* pStream,
     pDict->HDPATS[GRAY] = BHDC->subImage(HDPW * GRAY, 0, HDPW, HDPH);
     GRAY = GRAY + 1;
   }
-  delete BHDC;
-  return pDict.release();
+  return pDict;
 }
