@@ -9,6 +9,7 @@
 #include <algorithm>
 #include <utility>
 
+#include "core/fxcrt/fx_stream.h"
 #include "core/fxcrt/xml/cfx_saxreaderhandler.h"
 #include "third_party/base/ptr_util.h"
 #include "third_party/base/stl_util.h"
@@ -28,7 +29,6 @@ enum class CFX_SaxMode {
   TagClose,
   TagEnd,
   TargetData,
-  MAX
 };
 
 class CFX_SAXCommentContext {
@@ -42,25 +42,6 @@ namespace {
 
 const uint32_t kSaxFileBufSize = 32768;
 
-typedef void (CFX_SAXReader::*FX_SAXReader_LPFParse)();
-static const FX_SAXReader_LPFParse
-    g_FX_SAXReader_LPFParse[static_cast<int>(CFX_SaxMode::MAX)] = {
-        &CFX_SAXReader::ParseText,
-        &CFX_SAXReader::ParseNodeStart,
-        &CFX_SAXReader::ParseDeclOrComment,
-        &CFX_SAXReader::ParseDeclNode,
-        &CFX_SAXReader::ParseComment,
-        &CFX_SAXReader::ParseCommentContent,
-        &CFX_SAXReader::ParseTagName,
-        &CFX_SAXReader::ParseTagAttributeName,
-        &CFX_SAXReader::ParseTagAttributeEqual,
-        &CFX_SAXReader::ParseTagAttributeValue,
-        &CFX_SAXReader::ParseMaybeClose,
-        &CFX_SAXReader::ParseTagClose,
-        &CFX_SAXReader::ParseTagEnd,
-        &CFX_SAXReader::ParseTargetData,
-};
-
 }  // namespace
 
 CFX_SAXFile::CFX_SAXFile()
@@ -73,7 +54,7 @@ CFX_SAXFile::CFX_SAXFile()
 
 CFX_SAXFile::~CFX_SAXFile() {}
 
-bool CFX_SAXFile::StartFile(const CFX_RetainPtr<IFX_SeekableReadStream>& pFile,
+bool CFX_SAXFile::StartFile(const RetainPtr<IFX_SeekableReadStream>& pFile,
                             uint32_t dwStart,
                             uint32_t dwLen) {
   ASSERT(!m_pFile && pFile);
@@ -127,36 +108,25 @@ CFX_SAXReader::CFX_SAXReader()
       m_pHandler(nullptr),
       m_iState(-1),
       m_dwItemID(0),
-      m_iDataSize(256),
-      m_iNameSize(256),
-      m_dwParseMode(0),
-      m_pCommentContext(nullptr) {
-  m_pszData = FX_Alloc(uint8_t, m_iDataSize);
-  m_pszName = FX_Alloc(uint8_t, m_iNameSize);
+      m_dwParseMode(0) {
+  m_Data.reserve(256);
+  m_Name.reserve(256);
 }
+
 CFX_SAXReader::~CFX_SAXReader() {
   Reset();
-  if (m_pszData) {
-    FX_Free(m_pszData);
-    m_pszData = nullptr;
-  }
-  if (m_pszName) {
-    FX_Free(m_pszName);
-    m_pszName = nullptr;
-  }
 }
 
 void CFX_SAXReader::Reset() {
   m_File.Reset();
+  m_iState = -1;
   m_Stack = std::stack<std::unique_ptr<CFX_SAXItem>>();
   m_dwItemID = 0;
   m_SkipStack = std::stack<char>();
   m_SkipChar = 0;
-  m_iDataLength = 0;
-  m_iEntityStart = -1;
-  m_iNameLength = 0;
-  m_iDataPos = 0;
   m_pCommentContext.reset();
+  ClearData();
+  ClearName();
 }
 
 void CFX_SAXReader::Push() {
@@ -176,38 +146,39 @@ CFX_SAXItem* CFX_SAXReader::GetCurrentItem() const {
   return m_Stack.empty() ? nullptr : m_Stack.top().get();
 }
 
-void CFX_SAXReader::AppendData(uint8_t ch) {
-  ReallocDataBuffer();
-  m_pszData[m_iDataPos++] = ch;
+void CFX_SAXReader::ClearData() {
+  m_Data.clear();
+  m_iEntityStart = -1;
 }
 
-void CFX_SAXReader::AppendName(uint8_t ch) {
-  ReallocNameBuffer();
-  m_pszName[m_iDataPos++] = ch;
+void CFX_SAXReader::ClearName() {
+  m_Name.clear();
 }
 
-void CFX_SAXReader::ReallocDataBuffer() {
-  if (m_iDataPos < m_iDataSize) {
-    return;
-  }
-  if (m_iDataSize <= 1024 * 1024) {
-    m_iDataSize *= 2;
-  } else {
-    m_iDataSize += 1024 * 1024;
-  }
-  m_pszData = (uint8_t*)FX_Realloc(uint8_t, m_pszData, m_iDataSize);
+void CFX_SAXReader::AppendToData(uint8_t ch) {
+  m_Data.push_back(ch);
 }
 
-void CFX_SAXReader::ReallocNameBuffer() {
-  if (m_iDataPos < m_iNameSize) {
-    return;
-  }
-  if (m_iNameSize <= 1024 * 1024) {
-    m_iNameSize *= 2;
-  } else {
-    m_iNameSize += 1024 * 1024;
-  }
-  m_pszName = (uint8_t*)FX_Realloc(uint8_t, m_pszName, m_iNameSize);
+void CFX_SAXReader::AppendToName(uint8_t ch) {
+  m_Name.push_back(ch);
+}
+
+void CFX_SAXReader::BackUpAndReplaceDataAt(int32_t index, uint8_t ch) {
+  ASSERT(index > -1);
+  m_Data.erase(m_Data.begin() + index, m_Data.end());
+  AppendToData(ch);
+}
+
+int32_t CFX_SAXReader::CurrentDataIndex() const {
+  return pdfium::CollectionSize<int32_t>(m_Data) - 1;
+}
+
+bool CFX_SAXReader::IsEntityStart(uint8_t ch) const {
+  return m_iEntityStart == -1 && ch == '&';
+}
+
+bool CFX_SAXReader::IsEntityEnd(uint8_t ch) const {
+  return m_iEntityStart != -1 && ch == ';';
 }
 
 bool CFX_SAXReader::SkipSpace(uint8_t ch) {
@@ -215,11 +186,10 @@ bool CFX_SAXReader::SkipSpace(uint8_t ch) {
 }
 
 int32_t CFX_SAXReader::StartParse(
-    const CFX_RetainPtr<IFX_SeekableReadStream>& pFile,
+    const RetainPtr<IFX_SeekableReadStream>& pFile,
     uint32_t dwStart,
     uint32_t dwLen,
     uint32_t dwParseMode) {
-  m_iState = -1;
   Reset();
   if (!m_File.StartFile(pFile, dwStart, dwLen))
     return -1;
@@ -230,135 +200,175 @@ int32_t CFX_SAXReader::StartParse(
   m_bCharData = false;
   m_dwDataOffset = 0;
   m_dwParseMode = dwParseMode;
-  m_Stack.emplace(new CFX_SAXItem(++m_dwItemID));
+  m_Stack.push(pdfium::MakeUnique<CFX_SAXItem>(++m_dwItemID));
   return 0;
 }
 
-int32_t CFX_SAXReader::ContinueParse(IFX_Pause* pPause) {
-  if (m_iState < 0 || m_iState > 99) {
+int32_t CFX_SAXReader::ContinueParse() {
+  if (m_iState < 0 || m_iState > 99)
     return m_iState;
-  }
+
   while (m_File.m_dwCur < m_File.m_dwEnd) {
     uint32_t& index = m_File.m_dwBufIndex;
     uint32_t size = m_File.m_dwBufSize;
     const uint8_t* pBuf = m_File.m_pBuf;
     while (index < size) {
       m_CurByte = pBuf[index];
-      (this->*g_FX_SAXReader_LPFParse[static_cast<int>(m_eMode)])();
+      ParseInternal();
       index++;
     }
     m_File.m_dwCur += index;
     m_iState = (m_File.m_dwCur - m_File.m_dwStart) * 100 /
                (m_File.m_dwEnd - m_File.m_dwStart);
-    if (m_File.m_dwCur >= m_File.m_dwEnd) {
+    if (m_File.m_dwCur >= m_File.m_dwEnd)
       break;
-    }
     if (!m_File.ReadNextBlock()) {
       m_iState = -2;
       break;
     }
     m_dwDataOffset = 0;
-    if (pPause && pPause->NeedToPauseNow()) {
-      break;
-    }
   }
   return m_iState;
 }
+
+void CFX_SAXReader::ParseInternal() {
+  switch (m_eMode) {
+    case CFX_SaxMode::Text:
+      ParseText();
+      break;
+    case CFX_SaxMode::NodeStart:
+      ParseNodeStart();
+      break;
+    case CFX_SaxMode::DeclOrComment:
+      ParseDeclOrComment();
+      break;
+    case CFX_SaxMode::DeclNode:
+      ParseDeclNode();
+      break;
+    case CFX_SaxMode::Comment:
+      ParseComment();
+      break;
+    case CFX_SaxMode::CommentContent:
+      ParseCommentContent();
+      break;
+    case CFX_SaxMode::TagName:
+      ParseTagName();
+      break;
+    case CFX_SaxMode::TagAttributeName:
+      ParseTagAttributeName();
+      break;
+    case CFX_SaxMode::TagAttributeEqual:
+      ParseTagAttributeEqual();
+      break;
+    case CFX_SaxMode::TagAttributeValue:
+      ParseTagAttributeValue();
+      break;
+    case CFX_SaxMode::TagMaybeClose:
+      ParseMaybeClose();
+      break;
+    case CFX_SaxMode::TagClose:
+      ParseTagClose();
+      break;
+    case CFX_SaxMode::TagEnd:
+      ParseTagEnd();
+      break;
+    case CFX_SaxMode::TargetData:
+      ParseTargetData();
+      break;
+  }
+}
+
 void CFX_SAXReader::ParseChar(uint8_t ch) {
-  ReallocDataBuffer();
-  m_pszData[m_iDataPos] = ch;
-  if (m_iEntityStart > -1 && ch == ';') {
-    int32_t iSaveEntityStart = m_iEntityStart;
-    CFX_ByteString csEntity(m_pszData + m_iEntityStart + 1,
-                            m_iDataPos - m_iEntityStart - 1);
-    int32_t iLen = csEntity.GetLength();
-    if (iLen > 0) {
-      if (csEntity[0] == '#') {
-        if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_sharp) == 0) {
-          ch = 0;
-          uint8_t w;
-          if (iLen > 1 && csEntity[1] == 'x') {
-            for (int32_t i = 2; i < iLen; i++) {
-              w = csEntity[i];
-              if (w >= '0' && w <= '9') {
-                ch = (ch << 4) + w - '0';
-              } else if (w >= 'A' && w <= 'F') {
-                ch = (ch << 4) + w - 55;
-              } else if (w >= 'a' && w <= 'f') {
-                ch = (ch << 4) + w - 87;
-              } else {
-                break;
-              }
-            }
-          } else {
-            for (int32_t i = 1; i < iLen; i++) {
-              w = csEntity[i];
-              if (w < '0' || w > '9') {
-                break;
-              }
-              ch = ch * 10 + w - '0';
-            }
-          }
-          if (ch != 0) {
-            m_pszData[m_iEntityStart++] = ch;
-          }
+  AppendToData(ch);
+  if (IsEntityStart(ch)) {
+    m_iEntityStart = CurrentDataIndex();
+    return;
+  }
+  if (!IsEntityEnd(ch))
+    return;
+
+  // No matter what, we're no longer in an entity.
+  ASSERT(m_iEntityStart > -1);
+  int32_t iSaveStart = m_iEntityStart;
+  m_iEntityStart = -1;
+
+  // NOTE: Relies on negative lengths being treated as empty strings.
+  ByteString csEntity(m_Data.data() + iSaveStart + 1,
+                      CurrentDataIndex() - iSaveStart - 1);
+  int32_t iLen = csEntity.GetLength();
+  if (iLen == 0)
+    return;
+
+  if (csEntity[0] == '#') {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_sharp) == 0) {
+      ch = 0;
+      uint8_t w;
+      if (iLen > 1 && csEntity[1] == 'x') {
+        for (int32_t i = 2; i < iLen; i++) {
+          w = csEntity[i];
+          if (w >= '0' && w <= '9')
+            ch = (ch << 4) + w - '0';
+          else if (w >= 'A' && w <= 'F')
+            ch = (ch << 4) + w - 55;
+          else if (w >= 'a' && w <= 'f')
+            ch = (ch << 4) + w - 87;
+          else
+            break;
         }
       } else {
-        if (csEntity.Compare("amp") == 0) {
-          if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_amp) == 0) {
-            m_pszData[m_iEntityStart++] = '&';
-          }
-        } else if (csEntity.Compare("lt") == 0) {
-          if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_lt) == 0) {
-            m_pszData[m_iEntityStart++] = '<';
-          }
-        } else if (csEntity.Compare("gt") == 0) {
-          if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_gt) == 0) {
-            m_pszData[m_iEntityStart++] = '>';
-          }
-        } else if (csEntity.Compare("apos") == 0) {
-          if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_apos) == 0) {
-            m_pszData[m_iEntityStart++] = '\'';
-          }
-        } else if (csEntity.Compare("quot") == 0) {
-          if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_quot) == 0) {
-            m_pszData[m_iEntityStart++] = '\"';
-          }
+        for (int32_t i = 1; i < iLen; i++) {
+          w = csEntity[i];
+          if (w < '0' || w > '9')
+            break;
+          ch = ch * 10 + w - '0';
         }
       }
+      if (ch != 0)
+        BackUpAndReplaceDataAt(iSaveStart, ch);
     }
-    if (iSaveEntityStart != m_iEntityStart) {
-      m_iDataPos = m_iEntityStart;
-      m_iEntityStart = -1;
-    } else {
-      m_iDataPos++;
-      m_iEntityStart = -1;
-    }
-  } else {
-    if (m_iEntityStart < 0 && ch == '&') {
-      m_iEntityStart = m_iDataPos;
-    }
-    m_iDataPos++;
+    return;
+  }
+  if (csEntity == "amp") {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_amp) == 0)
+      BackUpAndReplaceDataAt(iSaveStart, '&');
+    return;
+  }
+  if (csEntity == "lt") {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_lt) == 0)
+      BackUpAndReplaceDataAt(iSaveStart, '<');
+    return;
+  }
+  if (csEntity == "gt") {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_gt) == 0)
+      BackUpAndReplaceDataAt(iSaveStart, '>');
+    return;
+  }
+  if (csEntity == "apos") {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_apos) == 0)
+      BackUpAndReplaceDataAt(iSaveStart, '\'');
+    return;
+  }
+  if (csEntity == "quot") {
+    if ((m_dwParseMode & CFX_SaxParseMode_NotConvert_quot) == 0)
+      BackUpAndReplaceDataAt(iSaveStart, '\"');
+    return;
   }
 }
 
 void CFX_SAXReader::ParseText() {
   if (m_CurByte == '<') {
-    if (m_iDataPos > 0) {
-      m_iDataLength = m_iDataPos;
-      m_iDataPos = 0;
-      if (m_pHandler) {
-        NotifyData();
-      }
+    if (!m_Data.empty()) {
+      NotifyData();
+      ClearData();
     }
     Push();
     m_dwNodePos = m_File.m_dwCur + m_File.m_dwBufIndex;
     m_eMode = CFX_SaxMode::NodeStart;
     return;
   }
-  if (m_iDataPos < 1 && SkipSpace(m_CurByte)) {
+  if (m_Data.empty() && SkipSpace(m_CurByte))
     return;
-  }
+
   ParseChar(m_CurByte);
 }
 
@@ -385,7 +395,7 @@ void CFX_SAXReader::ParseNodeStart() {
     m_dwDataOffset = m_File.m_dwBufIndex;
     GetCurrentItem()->m_eNode = CFX_SAXItem::Type::Tag;
     m_eMode = CFX_SaxMode::TagName;
-    AppendData(m_CurByte);
+    AppendToData(m_CurByte);
   }
 }
 
@@ -415,58 +425,55 @@ void CFX_SAXReader::ParseComment() {
 void CFX_SAXReader::ParseCommentContent() {
   if (m_CurByte == '-') {
     m_pCommentContext->m_iTailCount++;
-  } else if (m_CurByte == '>' && m_pCommentContext->m_iTailCount == 2) {
-    m_iDataLength = m_iDataPos;
-    m_iDataPos = 0;
-    if (m_pHandler) {
-      NotifyTargetData();
-    }
+    return;
+  }
+  if (m_CurByte == '>' && m_pCommentContext->m_iTailCount == 2) {
+    NotifyTargetData();
+    ClearData();
     Pop();
     m_eMode = CFX_SaxMode::Text;
-  } else {
-    while (m_pCommentContext->m_iTailCount > 0) {
-      AppendData('-');
-      m_pCommentContext->m_iTailCount--;
-    }
-    AppendData(m_CurByte);
+    return;
   }
+  while (m_pCommentContext->m_iTailCount > 0) {
+    AppendToData('-');
+    m_pCommentContext->m_iTailCount--;
+  }
+  AppendToData(m_CurByte);
 }
+
 void CFX_SAXReader::ParseDeclNode() {
   SkipNode();
 }
+
 void CFX_SAXReader::ParseTagName() {
   if (m_CurByte < 0x21 || m_CurByte == '/' || m_CurByte == '>' ||
       m_CurByte == '?') {
-    m_iDataLength = m_iDataPos;
-    m_iDataPos = 0;
-    if (m_pHandler) {
-      NotifyEnter();
-    }
+    NotifyEnter();
+    ClearData();
     if (m_CurByte < 0x21) {
+      ClearName();
       m_eMode = CFX_SaxMode::TagAttributeName;
     } else if (m_CurByte == '/' || m_CurByte == '?') {
       m_ePrevMode = m_eMode;
       m_eMode = CFX_SaxMode::TagMaybeClose;
     } else {
-      if (m_pHandler) {
-        NotifyBreak();
-      }
+      NotifyBreak();
       m_eMode = CFX_SaxMode::Text;
     }
   } else {
-    AppendData(m_CurByte);
+    AppendToData(m_CurByte);
   }
 }
+
 void CFX_SAXReader::ParseTagAttributeName() {
   if (m_CurByte < 0x21 || m_CurByte == '=') {
-    if (m_iDataPos < 1 && m_CurByte < 0x21) {
+    if (m_Name.empty() && m_CurByte < 0x21)
       return;
-    }
-    m_iNameLength = m_iDataPos;
-    m_iDataPos = 0;
+
     m_SkipChar = 0;
     m_eMode = m_CurByte == '=' ? CFX_SaxMode::TagAttributeValue
                                : CFX_SaxMode::TagAttributeEqual;
+    ClearData();
     return;
   }
   if (m_CurByte == '/' || m_CurByte == '>' || m_CurByte == '?') {
@@ -474,17 +481,14 @@ void CFX_SAXReader::ParseTagAttributeName() {
       m_ePrevMode = m_eMode;
       m_eMode = CFX_SaxMode::TagMaybeClose;
     } else {
-      if (m_pHandler) {
-        NotifyBreak();
-      }
+      NotifyBreak();
       m_eMode = CFX_SaxMode::Text;
     }
     return;
   }
-  if (m_iDataPos < 1) {
+  if (m_Name.empty())
     m_dwDataOffset = m_File.m_dwBufIndex;
-  }
-  AppendName(m_CurByte);
+  AppendToName(m_CurByte);
 }
 
 void CFX_SAXReader::ParseTagAttributeEqual() {
@@ -494,8 +498,7 @@ void CFX_SAXReader::ParseTagAttributeEqual() {
     return;
   }
   if (GetCurrentItem()->m_eNode == CFX_SAXItem::Type::Instruction) {
-    m_iDataPos = m_iNameLength;
-    AppendName(0x20);
+    AppendToName(0x20);
     m_eMode = CFX_SaxMode::TargetData;
     ParseTargetData();
   }
@@ -504,13 +507,9 @@ void CFX_SAXReader::ParseTagAttributeEqual() {
 void CFX_SAXReader::ParseTagAttributeValue() {
   if (m_SkipChar) {
     if (m_SkipChar == m_CurByte) {
-      {
-        m_iDataLength = m_iDataPos;
-        m_iDataPos = 0;
-        if (m_pHandler) {
-          NotifyAttribute();
-        }
-      }
+      NotifyAttribute();
+      ClearData();
+      ClearName();
       m_SkipChar = 0;
       m_eMode = CFX_SaxMode::TagAttributeName;
       return;
@@ -521,36 +520,33 @@ void CFX_SAXReader::ParseTagAttributeValue() {
   if (m_CurByte < 0x21) {
     return;
   }
-  if (m_iDataPos < 1) {
-    if (m_CurByte == '\'' || m_CurByte == '\"') {
+  if (m_Data.empty()) {
+    if (m_CurByte == '\'' || m_CurByte == '\"')
       m_SkipChar = m_CurByte;
-    }
   }
 }
 
 void CFX_SAXReader::ParseMaybeClose() {
   if (m_CurByte == '>') {
     if (GetCurrentItem()->m_eNode == CFX_SAXItem::Type::Instruction) {
-      m_iNameLength = m_iDataPos;
-      m_iDataPos = 0;
-      if (m_pHandler) {
-        NotifyTargetData();
-      }
+      NotifyTargetData();
+      ClearData();
+      ClearName();
     }
     ParseTagClose();
     m_eMode = CFX_SaxMode::Text;
   } else if (m_ePrevMode == CFX_SaxMode::TagName) {
-    AppendData('/');
+    AppendToData('/');
     m_eMode = CFX_SaxMode::TagName;
     m_ePrevMode = CFX_SaxMode::Text;
     ParseTagName();
   } else if (m_ePrevMode == CFX_SaxMode::TagAttributeName) {
-    AppendName('/');
+    AppendToName('/');
     m_eMode = CFX_SaxMode::TagAttributeName;
     m_ePrevMode = CFX_SaxMode::Text;
     ParseTagAttributeName();
   } else if (m_ePrevMode == CFX_SaxMode::TargetData) {
-    AppendName('?');
+    AppendToName('?');
     m_eMode = CFX_SaxMode::TargetData;
     m_ePrevMode = CFX_SaxMode::Text;
     ParseTargetData();
@@ -558,9 +554,7 @@ void CFX_SAXReader::ParseMaybeClose() {
 }
 void CFX_SAXReader::ParseTagClose() {
   m_dwNodePos = m_File.m_dwCur + m_File.m_dwBufIndex;
-  if (m_pHandler) {
-    NotifyClose();
-  }
+  NotifyClose();
   Pop();
 }
 void CFX_SAXReader::ParseTagEnd() {
@@ -570,11 +564,8 @@ void CFX_SAXReader::ParseTagEnd() {
   if (m_CurByte == '>') {
     Pop();
     m_dwNodePos = m_File.m_dwCur + m_File.m_dwBufIndex;
-    m_iDataLength = m_iDataPos;
-    m_iDataPos = 0;
-    if (m_pHandler) {
-      NotifyEnd();
-    }
+    NotifyEnd();
+    ClearData();
     Pop();
     m_eMode = CFX_SaxMode::Text;
   } else {
@@ -586,7 +577,7 @@ void CFX_SAXReader::ParseTargetData() {
     m_ePrevMode = m_eMode;
     m_eMode = CFX_SaxMode::TagMaybeClose;
   } else {
-    AppendName(m_CurByte);
+    AppendToName(m_CurByte);
   }
 }
 void CFX_SAXReader::SkipNode() {
@@ -625,24 +616,18 @@ void CFX_SAXReader::SkipNode() {
         m_SkipStack.pop();
         m_SkipChar = !m_SkipStack.empty() ? m_SkipStack.top() : 0;
         if (m_SkipStack.empty() && m_CurByte == '>') {
-          m_iDataLength = m_iDataPos;
-          m_iDataPos = 0;
-          if (m_iDataLength >= 9 &&
-              memcmp(m_pszData, "[CDATA[", 7 * sizeof(uint8_t)) == 0 &&
-              memcmp(m_pszData + m_iDataLength - 2, "]]",
-                     2 * sizeof(uint8_t)) == 0) {
+          if (m_Data.size() >= 9 && memcmp(m_Data.data(), "[CDATA[", 7) == 0 &&
+              memcmp(m_Data.data() + m_Data.size() - 2, "]]", 2) == 0) {
             Pop();
-            m_iDataLength -= 9;
-            m_dwDataOffset += 7;
-            memmove(m_pszData, m_pszData + 7, m_iDataLength * sizeof(uint8_t));
+            m_Data.erase(m_Data.begin(), m_Data.begin() + 7);
+            m_Data.erase(m_Data.end() - 2, m_Data.end());
             m_bCharData = true;
-            if (m_pHandler) {
-              NotifyData();
-            }
+            NotifyData();
             m_bCharData = false;
           } else {
             Pop();
           }
+          ClearData();
           m_eMode = CFX_SaxMode::Text;
         }
       }
@@ -653,6 +638,9 @@ void CFX_SAXReader::SkipNode() {
 }
 
 void CFX_SAXReader::NotifyData() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
   if (!pItem)
     return;
@@ -661,37 +649,59 @@ void CFX_SAXReader::NotifyData() {
     m_pHandler->OnTagData(
         pItem->m_pNode,
         m_bCharData ? CFX_SAXItem::Type::CharData : CFX_SAXItem::Type::Text,
-        CFX_ByteStringC(m_pszData, m_iDataLength),
-        m_File.m_dwCur + m_dwDataOffset);
+        ByteStringView(m_Data), m_File.m_dwCur + m_dwDataOffset);
 }
 
 void CFX_SAXReader::NotifyEnter() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
   if (pItem->m_eNode == CFX_SAXItem::Type::Tag ||
       pItem->m_eNode == CFX_SAXItem::Type::Instruction) {
-    pItem->m_pNode = m_pHandler->OnTagEnter(
-        CFX_ByteStringC(m_pszData, m_iDataLength), pItem->m_eNode, m_dwNodePos);
+    pItem->m_pNode = m_pHandler->OnTagEnter(ByteStringView(m_Data),
+                                            pItem->m_eNode, m_dwNodePos);
   }
 }
 
 void CFX_SAXReader::NotifyAttribute() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
   if (pItem->m_eNode == CFX_SAXItem::Type::Tag ||
       pItem->m_eNode == CFX_SAXItem::Type::Instruction) {
-    m_pHandler->OnTagAttribute(pItem->m_pNode,
-                               CFX_ByteStringC(m_pszName, m_iNameLength),
-                               CFX_ByteStringC(m_pszData, m_iDataLength));
+    m_pHandler->OnTagAttribute(pItem->m_pNode, ByteStringView(m_Name),
+                               ByteStringView(m_Data));
   }
 }
 
 void CFX_SAXReader::NotifyBreak() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
   if (pItem->m_eNode == CFX_SAXItem::Type::Tag)
     m_pHandler->OnTagBreak(pItem->m_pNode);
 }
 
 void CFX_SAXReader::NotifyClose() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
   if (pItem->m_eNode == CFX_SAXItem::Type::Tag ||
       pItem->m_eNode == CFX_SAXItem::Type::Instruction) {
     m_pHandler->OnTagClose(pItem->m_pNode, m_dwNodePos);
@@ -699,31 +709,36 @@ void CFX_SAXReader::NotifyClose() {
 }
 
 void CFX_SAXReader::NotifyEnd() {
-  CFX_SAXItem* pItem = GetCurrentItem();
-  if (!pItem || pItem->m_eNode != CFX_SAXItem::Type::Tag)
+  if (!m_pHandler)
     return;
 
-  m_pHandler->OnTagEnd(pItem->m_pNode,
-                       CFX_ByteStringC(m_pszData, m_iDataLength), m_dwNodePos);
+  CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
+  if (pItem->m_eNode == CFX_SAXItem::Type::Tag)
+    m_pHandler->OnTagEnd(pItem->m_pNode, ByteStringView(m_Data), m_dwNodePos);
 }
 
 void CFX_SAXReader::NotifyTargetData() {
+  if (!m_pHandler)
+    return;
+
   CFX_SAXItem* pItem = GetCurrentItem();
+  if (!pItem)
+    return;
+
   if (pItem->m_eNode == CFX_SAXItem::Type::Instruction) {
     m_pHandler->OnTargetData(pItem->m_pNode, pItem->m_eNode,
-                             CFX_ByteStringC(m_pszName, m_iNameLength),
-                             m_dwNodePos);
+                             ByteStringView(m_Name), m_dwNodePos);
   } else if (pItem->m_eNode == CFX_SAXItem::Type::Comment) {
     m_pHandler->OnTargetData(pItem->m_pNode, pItem->m_eNode,
-                             CFX_ByteStringC(m_pszData, m_iDataLength),
-                             m_dwNodePos);
+                             ByteStringView(m_Data), m_dwNodePos);
   }
 }
 
 void CFX_SAXReader::SkipCurrentNode() {
   CFX_SAXItem* pItem = GetCurrentItem();
-  if (!pItem)
-    return;
-
-  pItem->m_bSkip = true;
+  if (pItem)
+    pItem->m_bSkip = true;
 }
