@@ -7,17 +7,20 @@
 #ifndef CORE_FXCRT_XML_CXML_CONTENT_H_
 #define CORE_FXCRT_XML_CXML_CONTENT_H_
 
-class CXML_Content {
- public:
-  CXML_Content() : m_bCDATA(false), m_Content() {}
+#include "core/fxcrt/fx_string.h"
+#include "core/fxcrt/xml/cxml_object.h"
 
-  void Set(bool bCDATA, const CFX_WideStringC& content) {
-    m_bCDATA = bCDATA;
-    m_Content = content;
-  }
+class CXML_Content : public CXML_Object {
+ public:
+  CXML_Content(bool bCDATA, const WideStringView& content);
+  ~CXML_Content() override;
+
+  // CXML_Object:
+  CXML_Content* AsContent() override;
+  const CXML_Content* AsContent() const override;
 
   bool m_bCDATA;
-  CFX_WideString m_Content;
+  WideString m_Content;
 };
 
 #endif  // CORE_FXCRT_XML_CXML_CONTENT_H_
