@@ -9,7 +9,8 @@
 
 #include <memory>
 
-#include "core/fxge/cfx_gemodule.h"
+#include "core/fxcrt/unowned_ptr.h"
+#include "core/fxge/cfx_color.h"
 #include "core/fxge/fx_dib.h"
 #include "core/fxge/fx_font.h"
 
@@ -30,7 +31,6 @@ class IFX_RenderDeviceDriver;
 
 #define FXRC_GET_BITS 0x01
 #define FXRC_BIT_MASK 0x02
-#define FXRC_ALPHA_MASK 0x04
 #define FXRC_ALPHA_PATH 0x10
 #define FXRC_ALPHA_IMAGE 0x20
 #define FXRC_ALPHA_OUTPUT 0x40
@@ -61,6 +61,8 @@ class IFX_RenderDeviceDriver;
 #define FXTEXT_PRINTIMAGETEXT 0x10
 #define FXTEXT_NOSMOOTH 0x20
 
+class CFX_PathData;
+
 enum class FXPT_TYPE : uint8_t { LineTo, BezierTo, MoveTo };
 
 class FXTEXT_CHARPOS {
@@ -71,9 +73,10 @@ class FXTEXT_CHARPOS {
 
   float m_AdjustMatrix[4];
   CFX_PointF m_Origin;
+  uint32_t m_Unicode;
   uint32_t m_GlyphIndex;
   int32_t m_FontCharWidth;
-#if _FXM_PLATFORM_ == _FXM_PLATFORM_APPLE_
+#if _FX_PLATFORM_ == _FX_PLATFORM_APPLE_
   uint32_t m_ExtGID;
 #endif
   int32_t m_FallbackFontPosition;
@@ -83,6 +86,15 @@ class FXTEXT_CHARPOS {
 
 class CFX_RenderDevice {
  public:
+  class StateRestorer {
+   public:
+    explicit StateRestorer(CFX_RenderDevice* pDevice);
+    ~StateRestorer();
+
+   private:
+    UnownedPtr<CFX_RenderDevice> m_pDevice;
+  };
+
   CFX_RenderDevice();
   virtual ~CFX_RenderDevice();
 
@@ -101,17 +113,18 @@ class CFX_RenderDevice {
   int GetRenderCaps() const { return m_RenderCaps; }
   int GetDeviceCaps(int id) const;
   CFX_Matrix GetCTM() const;
-  CFX_RetainPtr<CFX_DIBitmap> GetBitmap() const { return m_pBitmap; }
-  void SetBitmap(const CFX_RetainPtr<CFX_DIBitmap>& pBitmap) {
+  RetainPtr<CFX_DIBitmap> GetBitmap() const { return m_pBitmap; }
+  void SetBitmap(const RetainPtr<CFX_DIBitmap>& pBitmap) {
     m_pBitmap = pBitmap;
   }
-  bool CreateCompatibleBitmap(const CFX_RetainPtr<CFX_DIBitmap>& pDIB,
+  bool CreateCompatibleBitmap(const RetainPtr<CFX_DIBitmap>& pDIB,
                               int width,
                               int height) const;
   const FX_RECT& GetClipBox() const { return m_ClipBox; }
   bool SetClip_PathFill(const CFX_PathData* pPathData,
                         const CFX_Matrix* pObject2Device,
                         int fill_mode);
+  bool SetClip_Rect(const CFX_RectF& pRect);
   bool SetClip_Rect(const FX_RECT& pRect);
   bool SetClip_PathStroke(const CFX_PathData* pPathData,
                           const CFX_Matrix* pObject2Device,
@@ -132,31 +145,20 @@ class CFX_RenderDevice {
                          uint32_t stroke_color,
                          int fill_mode,
                          int blend_type);
-  bool SetPixel(int x, int y, uint32_t color);
   bool FillRect(const FX_RECT* pRect, uint32_t color) {
     return FillRectWithBlend(pRect, color, FXDIB_BLEND_NORMAL);
   }
-  bool FillRectWithBlend(const FX_RECT* pRect, uint32_t color, int blend_type);
-  bool DrawCosmeticLine(float x1,
-                        float y1,
-                        float x2,
-                        float y2,
-                        uint32_t color,
-                        int fill_mode,
-                        int blend_type);
 
-  CFX_RetainPtr<CFX_DIBitmap> GetBackDrop();
-  bool GetDIBits(const CFX_RetainPtr<CFX_DIBitmap>& pBitmap, int left, int top);
-  bool SetDIBits(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
-                 int left,
-                 int top) {
+  RetainPtr<CFX_DIBitmap> GetBackDrop();
+  bool GetDIBits(const RetainPtr<CFX_DIBitmap>& pBitmap, int left, int top);
+  bool SetDIBits(const RetainPtr<CFX_DIBSource>& pBitmap, int left, int top) {
     return SetDIBitsWithBlend(pBitmap, left, top, FXDIB_BLEND_NORMAL);
   }
-  bool SetDIBitsWithBlend(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool SetDIBitsWithBlend(const RetainPtr<CFX_DIBSource>& pBitmap,
                           int left,
                           int top,
                           int blend_type);
-  bool StretchDIBits(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StretchDIBits(const RetainPtr<CFX_DIBSource>& pBitmap,
                      int left,
                      int top,
                      int dest_width,
@@ -164,32 +166,31 @@ class CFX_RenderDevice {
     return StretchDIBitsWithFlagsAndBlend(pBitmap, left, top, dest_width,
                                           dest_height, 0, FXDIB_BLEND_NORMAL);
   }
-  bool StretchDIBitsWithFlagsAndBlend(
-      const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
-      int left,
-      int top,
-      int dest_width,
-      int dest_height,
-      uint32_t flags,
-      int blend_type);
-  bool SetBitMask(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StretchDIBitsWithFlagsAndBlend(const RetainPtr<CFX_DIBSource>& pBitmap,
+                                      int left,
+                                      int top,
+                                      int dest_width,
+                                      int dest_height,
+                                      uint32_t flags,
+                                      int blend_type);
+  bool SetBitMask(const RetainPtr<CFX_DIBSource>& pBitmap,
                   int left,
                   int top,
                   uint32_t color);
-  bool StretchBitMask(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StretchBitMask(const RetainPtr<CFX_DIBSource>& pBitmap,
                       int left,
                       int top,
                       int dest_width,
                       int dest_height,
                       uint32_t color);
-  bool StretchBitMaskWithFlags(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StretchBitMaskWithFlags(const RetainPtr<CFX_DIBSource>& pBitmap,
                                int left,
                                int top,
                                int dest_width,
                                int dest_height,
                                uint32_t color,
                                uint32_t flags);
-  bool StartDIBits(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StartDIBits(const RetainPtr<CFX_DIBSource>& pBitmap,
                    int bitmap_alpha,
                    uint32_t color,
                    const CFX_Matrix* pMatrix,
@@ -198,14 +199,14 @@ class CFX_RenderDevice {
     return StartDIBitsWithBlend(pBitmap, bitmap_alpha, color, pMatrix, flags,
                                 handle, FXDIB_BLEND_NORMAL);
   }
-  bool StartDIBitsWithBlend(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
+  bool StartDIBitsWithBlend(const RetainPtr<CFX_DIBSource>& pBitmap,
                             int bitmap_alpha,
                             uint32_t color,
                             const CFX_Matrix* pMatrix,
                             uint32_t flags,
                             std::unique_ptr<CFX_ImageRenderer>* handle,
                             int blend_type);
-  bool ContinueDIBits(CFX_ImageRenderer* handle, IFX_Pause* pPause);
+  bool ContinueDIBits(CFX_ImageRenderer* handle, IFX_PauseIndicator* pPause);
 
   bool DrawNormalText(int nChars,
                       const FXTEXT_CHARPOS* pCharPos,
@@ -226,17 +227,53 @@ class CFX_RenderDevice {
                     CFX_PathData* pClippingPath,
                     int nFlag);
 
+  void DrawFillRect(const CFX_Matrix* pUser2Device,
+                    const CFX_FloatRect& rect,
+                    const CFX_Color& color,
+                    int32_t nTransparency);
+  void DrawFillRect(const CFX_Matrix* pUser2Device,
+                    const CFX_FloatRect& rect,
+                    const FX_COLORREF& color);
+  void DrawStrokeRect(const CFX_Matrix* pUser2Device,
+                      const CFX_FloatRect& rect,
+                      const FX_COLORREF& color,
+                      float fWidth);
+  void DrawStrokeLine(const CFX_Matrix* pUser2Device,
+                      const CFX_PointF& ptMoveTo,
+                      const CFX_PointF& ptLineTo,
+                      const FX_COLORREF& color,
+                      float fWidth);
+  void DrawBorder(const CFX_Matrix* pUser2Device,
+                  const CFX_FloatRect& rect,
+                  float fWidth,
+                  const CFX_Color& color,
+                  const CFX_Color& crLeftTop,
+                  const CFX_Color& crRightBottom,
+                  BorderStyle nStyle,
+                  int32_t nTransparency);
+  void DrawFillArea(const CFX_Matrix* pUser2Device,
+                    const CFX_PointF* pPts,
+                    int32_t nCount,
+                    const FX_COLORREF& color);
+  void DrawShadow(const CFX_Matrix* pUser2Device,
+                  bool bVertical,
+                  bool bHorizontal,
+                  CFX_FloatRect rect,
+                  int32_t nTransparency,
+                  int32_t nStartGray,
+                  int32_t nEndGray);
+
 #ifdef _SKIA_SUPPORT_
   virtual void DebugVerifyBitmapIsPreMultiplied() const;
-  virtual bool SetBitsWithMask(const CFX_RetainPtr<CFX_DIBSource>& pBitmap,
-                               const CFX_RetainPtr<CFX_DIBSource>& pMask,
+  virtual bool SetBitsWithMask(const RetainPtr<CFX_DIBSource>& pBitmap,
+                               const RetainPtr<CFX_DIBSource>& pMask,
                                int left,
                                int top,
                                int bitmap_alpha,
                                int blend_type);
 #endif
 #if defined _SKIA_SUPPORT_ || defined _SKIA_SUPPORT_PATHS_
-  void Flush();
+  void Flush(bool release);
 #endif
 
  private:
@@ -249,8 +286,14 @@ class CFX_RenderDevice {
                           uint32_t stroke_color,
                           int fill_mode,
                           int blend_type);
+  bool DrawCosmeticLine(const CFX_PointF& ptMoveTo,
+                        const CFX_PointF& ptLineTo,
+                        uint32_t color,
+                        int fill_mode,
+                        int blend_type);
+  bool FillRectWithBlend(const FX_RECT* pRect, uint32_t color, int blend_type);
 
-  CFX_RetainPtr<CFX_DIBitmap> m_pBitmap;
+  RetainPtr<CFX_DIBitmap> m_pBitmap;
   int m_Width;
   int m_Height;
   int m_bpp;
