@@ -13,27 +13,26 @@
 #include "core/fxge/fx_dib.h"
 #include "fxbarcode/cbc_onecode.h"
 
+class CBC_OnedCode39Writer;
+
 class CBC_Code39 : public CBC_OneCode {
  public:
   CBC_Code39();
   ~CBC_Code39() override;
 
   // CBC_OneCode:
-  bool Encode(const CFX_WideStringC& contents,
-              bool isDevice,
-              int32_t& e) override;
+  bool Encode(const WideStringView& contents) override;
   bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix,
-                    int32_t& e) override;
-  bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                    int32_t& e) override;
+                    const CFX_Matrix* matrix) override;
   BC_TYPE GetType() override;
 
   bool SetTextLocation(BC_TEXT_LOC location);
-  bool SetWideNarrowRatio(int32_t ratio);
+  bool SetWideNarrowRatio(int8_t ratio);
 
  private:
-  CFX_WideString m_renderContents;
+  CBC_OnedCode39Writer* GetOnedCode39Writer();
+
+  WideString m_renderContents;
 };
 
 #endif  // FXBARCODE_CBC_CODE39_H_
