@@ -186,14 +186,14 @@ bool Bookmark::read(StarZone &zone)
     zone.closeFlagZone();
   }
   if (ok && input->tell()<zone.getRecordLastPosition()) {
-    for (int i=0; i<4; ++i) { // start[aMac:aLib],end[aMac:Alib]
+    for (auto &macroName : m_macroNames) {  // start[aMac:aLib],end[aMac:Alib]
       if (!zone.readString(text)) {
         STOFF_DEBUG_MSG(("StarWriterStruct::Bookmark::read: can not read macro name\n"));
         f << "###macro";
         break;
       }
       else
-        m_macroNames[i]=libstoff::getString(text);
+        macroName=libstoff::getString(text);
     }
   }
 
@@ -327,7 +327,7 @@ bool DatabaseName::read(StarZone &zone)
         }
         data.m_name=libstoff::getString(text);
         int positions[2];
-        for (int j=0; j<2; ++j) positions[i]=int(input->readULong(4));
+        for (int &position : positions) position=int(input->readULong(4));
         data.m_selection=STOFFVec2i(positions[0],positions[1]);
         m_dataList.push_back(data);
       }
@@ -350,8 +350,8 @@ std::ostream &operator<<(std::ostream &o, DatabaseName const &dbase)
   if (!dbase.m_sql.empty()) o << "sql=" << dbase.m_sql.cstr() << ",";
   if (!dbase.m_dataList.empty()) {
     o << "data=[";
-    for (size_t i=0; i<dbase.m_dataList.size(); ++i)
-      o << "[" << dbase.m_dataList[i] << "],";
+    for (auto const &data : dbase.m_dataList)
+      o << "[" << data << "],";
     o << "],";
   }
   return o;
@@ -402,8 +402,8 @@ std::ostream &operator<<(std::ostream &o, Dictionary const &dbase)
 {
   if (!dbase.m_dataList.empty()) {
     o << "data=[";
-    for (size_t i=0; i<dbase.m_dataList.size(); ++i)
-      o << "[" << dbase.m_dataList[i] << "],";
+    for (auto const &data : dbase.m_dataList)
+      o << "[" << data << "],";
     o << "],";
   }
   return o;
@@ -473,7 +473,7 @@ bool Macro::read(StarZone &zone)
   f << "Entries(StarMacro)[" << zone.getRecordLevel() << "]:";
   m_key=int(input->readULong(2));
   bool ok=true;
-  for (int i=0; i<2; ++i) {
+  for (auto &name : m_names) {
     std::vector<uint32_t> string;
     if (!zone.readString(string)) {
       STOFF_DEBUG_MSG(("StarObjectText::readSWMacroTable: can not read a string\n"));
@@ -481,7 +481,7 @@ bool Macro::read(StarZone &zone)
       ok=false;
       break;
     }
-    m_names[i]=libstoff::getString(string);
+    name=libstoff::getString(string);
   }
   if (ok && zone.isCompatibleWith(0x102))
     m_scriptType=int(input->readULong(2));
@@ -732,9 +732,9 @@ bool PrintData::read(StarZone &zone)
   f << "Entries(StarPrintData)[" << zone.getRecordLevel() << "]:";
   m_flags=int(input->readULong(1));
   int dim[2];
-  for (int i=0; i<2; ++i) dim[i]=int(input->readULong(2));
+  for (int &i : dim) i=int(input->readULong(2));
   m_colRow=STOFFVec2i(dim[1],dim[0]);
-  for (int i=0; i<6; ++i) m_spacings[i]=int(input->readULong(2));
+  for (int &spacing : m_spacings) spacing=int(input->readULong(2));
   f << *this;
   ascFile.addPos(pos);
   ascFile.addNote(f.str().c_str());
@@ -988,7 +988,7 @@ bool TOX::read(StarZone &zone, StarObject &object)
 
   if ((fl&0x10)) {
     while (input->tell()<lastRecordPos && input->peek()=='s') {
-      shared_ptr<StarFormatManagerInternal::FormatDef> format;
+      std::shared_ptr<StarFormatManagerInternal::FormatDef> format;
       if (!object.getFormatManager()->readSWFormatDef(zone,'s', format, object)) {
         STOFF_DEBUG_MSG(("StarWriterStruct::TOX::read: can not read some format\n"));
         f << "###format,";
@@ -1054,18 +1054,18 @@ std::ostream &operator<<(std::ostream &o, TOX const &tox)
   if (tox.m_OLEOptions) o << "OLEOptions=" << tox.m_OLEOptions << ",";
   if (!tox.m_stringIdList.empty()) {
     o << "stringIdList=[";
-    for (size_t i=0; i<tox.m_stringIdList.size(); ++i) {
-      if (tox.m_stringIdList[i]==0xFFFF)
+    for (auto const &id : tox.m_stringIdList) {
+      if (id==0xFFFF)
         o << "_,";
       else
-        o << tox.m_stringIdList[i] << ",";
+        o << id << ",";
     }
     o << "],";
   }
   if (!tox.m_styleList.empty()) {
     o << "styleList=[";
-    for (size_t i=0; i<tox.m_styleList.size(); ++i)
-      o << "[" << tox.m_styleList[i] << "],";
+    for (auto const &style : tox.m_styleList)
+      o << "[" << style << "],";
     o << "],";
   }
   if (tox.m_titleLength) o << "titleLength=" << tox.m_titleLength << ",";
@@ -1197,17 +1197,17 @@ std::ostream &operator<<(std::ostream &o, TOX51 const &tox)
   if (!tox.m_title.empty()) o << "title=" << tox.m_title.cstr() << ",";
   if (!tox.m_patternList.empty()) {
     o << "patternList=[";
-    for (size_t i=0; i<tox.m_patternList.size(); ++i)
-      o << tox.m_patternList[i].cstr() << ",";
+    for (auto const &pattern : tox.m_patternList)
+      o << pattern.cstr() << ",";
     o << "],";
   }
   if (!tox.m_stringIdList.empty()) {
     o << "stringIdList=[";
-    for (size_t i=0; i<tox.m_stringIdList.size(); ++i) {
-      if (tox.m_stringIdList[i]==0xFFFF)
+    for (auto const &id : tox.m_stringIdList) {
+      if (id==0xFFFF)
         o << "_,";
       else
-        o << tox.m_stringIdList[i] << ",";
+        o << id << ",";
     }
     o << "],";
   }
