@@ -8,13 +8,13 @@
 #define XFA_FWL_CFWL_NOTEDRIVER_H_
 
 #include <deque>
+#include <map>
 #include <memory>
-#include <unordered_map>
 #include <vector>
 
 #include "xfa/fwl/cfwl_event.h"
 #include "xfa/fwl/cfwl_widget.h"
-#include "xfa/fxgraphics/cfx_graphics.h"
+#include "xfa/fxgraphics/cxfa_graphics.h"
 
 class CFWL_EventTarget;
 class CFWL_NoteLoop;
@@ -30,7 +30,7 @@ class CFWL_NoteDriver {
 
   void RegisterEventTarget(CFWL_Widget* pListener, CFWL_Widget* pEventSource);
   void UnregisterEventTarget(CFWL_Widget* pListener);
-  void ClearEventTargets(bool bRemoveAll);
+  void ClearEventTargets();
 
   CFWL_NoteLoop* GetTopLoop() const;
   void PushNoteLoop(CFWL_NoteLoop* pNoteLoop);
@@ -69,7 +69,7 @@ class CFWL_NoteDriver {
   std::vector<CFWL_Widget*> m_Forms;
   std::deque<std::unique_ptr<CFWL_Message>> m_NoteQueue;
   std::vector<CFWL_NoteLoop*> m_NoteLoopQueue;
-  std::unordered_map<uint32_t, CFWL_EventTarget*> m_eventTargets;
+  std::map<uint32_t, std::unique_ptr<CFWL_EventTarget>> m_eventTargets;
   CFWL_Widget* m_pHover;
   CFWL_Widget* m_pFocus;
   CFWL_Widget* m_pGrab;
