@@ -8,12 +8,11 @@
 
 #include <vector>
 
-#include "xfa/fxfa/app/xfa_ffchoicelist.h"
-#include "xfa/fxfa/app/xfa_fffield.h"
-#include "xfa/fxfa/app/xfa_fwladapter.h"
 #include "xfa/fxfa/cxfa_ffdoc.h"
 #include "xfa/fxfa/cxfa_ffdocview.h"
+#include "xfa/fxfa/cxfa_fffield.h"
 #include "xfa/fxfa/cxfa_ffwidget.h"
+#include "xfa/fxfa/cxfa_fwladapterwidgetmgr.h"
 #include "xfa/fxfa/parser/cxfa_layoutprocessor.h"
 #include "xfa/fxfa/parser/cxfa_measurement.h"
 #include "xfa/fxfa/parser/cxfa_node.h"
@@ -143,6 +142,23 @@ bool CXFA_FFWidgetHandler::OnChar(CXFA_FFWidget* hWidget,
   return bRet;
 }
 
+WideString CXFA_FFWidgetHandler::GetSelectedText(CXFA_FFWidget* widget) {
+  if (!widget->CanCopy())
+    return WideString();
+
+  WideString val;
+  widget->Copy(val);
+  return val;
+}
+
+void CXFA_FFWidgetHandler::PasteText(CXFA_FFWidget* widget,
+                                     const WideString& text) {
+  if (!widget->CanPaste())
+    return;
+
+  widget->Paste(text);
+}
+
 FWL_WidgetHit CXFA_FFWidgetHandler::OnHitTest(CXFA_FFWidget* hWidget,
                                               const CFX_PointF& point) {
   if (!(hWidget->GetStatus() & XFA_WidgetStatus_Visible))
@@ -156,10 +172,10 @@ bool CXFA_FFWidgetHandler::OnSetCursor(CXFA_FFWidget* hWidget,
 }
 
 void CXFA_FFWidgetHandler::RenderWidget(CXFA_FFWidget* hWidget,
-                                        CFX_Graphics* pGS,
-                                        CFX_Matrix* pMatrix,
+                                        CXFA_Graphics* pGS,
+                                        const CFX_Matrix& matrix,
                                         bool bHighlight) {
-  hWidget->RenderWidget(pGS, pMatrix,
+  hWidget->RenderWidget(pGS, matrix,
                         bHighlight ? XFA_WidgetStatus_Highlight : 0);
 }
 
@@ -177,13 +193,14 @@ bool CXFA_FFWidgetHandler::HasEvent(CXFA_WidgetAcc* pWidgetAcc,
       return calc && calc.GetScript();
     }
     case XFA_EVENT_Validate: {
-      CXFA_Validate val = pWidgetAcc->GetValidate();
+      CXFA_Validate val = pWidgetAcc->GetValidate(false);
       return val && val.GetScript();
     }
     default:
       break;
   }
-  return !pWidgetAcc->GetEventByActivity(gs_EventActivity[eEventType]).empty();
+  return !pWidgetAcc->GetEventByActivity(gs_EventActivity[eEventType], false)
+              .empty();
 }
 
 int32_t CXFA_FFWidgetHandler::ProcessEvent(CXFA_WidgetAcc* pWidgetAcc,
